
#include "raptormenuwidget.h"

#include <QDebug>
#include <QGraphicsScene>
#include <QWheelEvent>
#include <kaboutdata.h>
#include <kcmdlineargs.h>
#include <klocale.h>
#include <KIcon>
#include <QGraphicsScene>
#include <kicontheme.h>
#include <kservicegroup.h>
#include <kservice.h>

//plsma
#include <QGraphicsRectItem>
#include <plasma/widgets/pushbutton.h>
#include <plasma/widgets/lineedit.h>
#include <plasma/widgets/icon.h>

//raptor
#include "bgelement.h"
#include <math.h>

class RaptorMenuWidget::Private
{
    public:
        Private(){};
        ~Private(){};
    
        QRectF boundingrect;
        RaptorCanvas *scene;
        QTimeLine timeline;
        QTimer * timer;
        int angle;
        Plasma::Svg * background;
        RaptorSliderViewGroup * view;
        RaptorSliderViewGroup * viewindex;
        RaptorPluginLoader * loader;
        BgElement * e;
	Plasma::LineEdit * lineedit;
};

RaptorMenuWidget::RaptorMenuWidget()
    : d(new Private),timerId(0),clicked(0)
{ 
    /*View Properties*/
    setSelfProperties();
    d->loader=0;
    /* Basic UI items*/
    d->view = new RaptorSliderViewGroup(d->scene);
    d->viewindex = new RaptorSliderViewGroup(d->scene);

    d->view->setItemLimit(4);
    d->view->setItemPerRow(2);
    d->boundingrect = this->sceneRect();
    d->e->setSize(sceneRect().width(),sceneRect().height());
    resize(sceneRect().width()+40,sceneRect().height()+60);
    d->viewindex->setItemLimit(4);
    d->viewindex->setItemPerRow(4);

    d->lineedit = new Plasma::LineEdit(d->e);
    d->lineedit->show();
    d->lineedit->setStyled(true);
    d->lineedit->setGeometry(QRect(300,20,300,10));
    //d->view->setMode(RaptorSlide::Grid);
    d->view->setMode(RaptorSlide::Horizontal);
    d->viewindex->setMode(RaptorSlide::Horizontal);

    /*Setup UI properties*/
    /* load data */
    loadDefaultPlugin();

    d->view->view(0);
    d->viewindex->view(0);
    //TODO
    ///read menu size

    d->boundingrect =
    QRectF(0,0,
            d->viewindex->getDefaultScene()->sceneRect().width()+20,
            d->view->getDefaultScene()->sceneRect().height() +
            d->viewindex->getDefaultScene()->sceneRect().height());
   /* Signal connections*/
    connect(d->scene,SIGNAL(sceneRectChanged(const QRectF &)),this,SLOT(update(const QRectF&)));

  //Plasma settup
   d->background = new Plasma::Svg( "widgets/background",this);

   //size 
   d->background->resize((int)d->view->getDefaultScene()->sceneRect
           ().width(),(int)d->view->getDefaultScene()->sceneRect ().height());

    move(10,10);

   //move
   d->view->move(25,20);
   d->viewindex->move(25,(d->view->height()*2)+50);



    d->boundingrect = this->sceneRect();
    resize(sceneRect().width()+60,sceneRect().height()+60);
    d->e->setSize(sceneRect().width(),sceneRect().height());
    //animation;
    d->timeline.setDuration(720);
    d->timeline.setFrameRange(0,360);
    d->timeline.setUpdateInterval(1000/100);
    connect(&d->timeline,SIGNAL(frameChanged(int)),this,SLOT(animate(int)));
    connect(&d->timeline,SIGNAL(finished()),this,SLOT(finishAnimation()));

        //kDebug()<<d->boundingrect<<endl;

}


void RaptorMenuWidget::setSelfProperties()
{
    setWindowFlags(Qt::FramelessWindowHint);
    d->scene = new RaptorCanvas();
    d->scene->setItemIndexMethod(QGraphicsScene::NoIndex);
    setScene(d->scene);
    setRenderHint(QPainter::Antialiasing);
    setTransformationAnchor(AnchorUnderMouse);
    setResizeAnchor(AnchorUnderMouse);
    setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    setLineWidth(0);
    setFrameStyle(QFrame::NoFrame);
    centerOn(QPointF(0.0,0.0));
    setAlignment(Qt::AlignLeft|Qt::AlignTop);
    d->e  = new BgElement();
    d->e->show();
    d->e->setPos(10,10);
    d->scene->addItem(d->e);

}

void RaptorMenuWidget::itemMoved()
{
    if (!timerId)
        timerId = startTimer(1000 / 25);
}
void RaptorMenuWidget::keyPressEvent(QKeyEvent *e)
{
    if (e->key() == Qt::Key_Left) {

    }
    if (e->key() == Qt::Key_Up) {
     
    }
    if ( e->key() == Qt::Key_W) {
//    timeline.stop();
        d->timeline.setDirection(QTimeLine::Forward);
        if (d->timeline.state() == QTimeLine::NotRunning)
        d->timeline.start();
    }
    if ( e->key() == Qt::Key_Q) {

    }
	e->ignore();
}


void RaptorMenuWidget::timerEvent(QTimerEvent *event)
{
    Q_UNUSED(event);

}

void RaptorMenuWidget::wheelEvent(QWheelEvent *event)
{
    scaleView(pow((double)2, -event->delta() / 240.0));
}

void RaptorMenuWidget::drawBackground(QPainter *painter, const QRectF &rect)
{

   Q_UNUSED(rect);
      painter->setCompositionMode(QPainter::CompositionMode_Source);
      painter->fillRect(QRect((int)rect.x(),(int)rect.y(),(int)rect.width()+2,(int)rect.height()+2), Qt::transparent);

      painter->setCompositionMode(QPainter::CompositionMode_SourceOver);

}

void RaptorMenuWidget::scaleView(qreal scaleFactor)
{
    qreal factor = matrix().scale(scaleFactor, scaleFactor).mapRect(QRectF(QCursor::pos().x(), 0, 1, 1)).width();
    if (factor < 0.07 || factor > 100)
    {
            return;
    }
    scale(scaleFactor, scaleFactor);

}

void RaptorMenuWidget::loadDefaultPlugin()
{
    d->loader= new RaptorPluginLoader();
    for (int i = 0; i < d->loader->pluginList()->size(); ++i) {
    RaptorClaw * item = new RaptorClaw(d->e);
    Raptorlet * rcore= d->loader->getDataService(d->loader->pluginList()->at(i));
    RaptorDataSourceGroup  * datagroup = NULL;
    if ( rcore != NULL)
    {
    item->setText(rcore->name());
    item->setName(d->loader->pluginList()->at(i));
    item->setIcon(KIcon ("applications-other").pixmap(64,64));
    d->viewindex->addItem(item);
    datagroup = rcore->data();
    }

        if (datagroup == NULL) {
       kDebug()<<"Data Group is null"<<endl;
        continue;
        }
        
/*
        if (datagroup->count() >= 0 ) {
            QStringList list = datagroup->getDataList();
            for (int i = 0 ; i < list.count() ; i++) {
             RaptorClaw * it = new RaptorClaw(d->e);
             it->setName(datagroup->getItem(list.at(i))->name() );
             it->setText(datagroup->getItem(list.at(i))->name() );
             it->setIcon(KIcon ("").pixmap(64,64));
             d->view->addItem(it);
             kDebug()<<datagroup->getItem(list.at(i))->name()<<endl;
            }
         }
*/
   //  connect(item,SIGNAL(clicked()),this,SLOT(loadView()));
     connect(item,SIGNAL(clicked(const QString&)),this,SLOT(loadView(const QString&)));

    }
  
}

void RaptorMenuWidget::loadView()
{
    d->view->deleteCurrent(); 
}

void RaptorMenuWidget::loadView(const QString& name)
{
    kDebug()<<name<<endl;
    d->view->deleteCurrent(); 
    if (d->loader !=0) {
        Raptorlet * raptorlet = d->loader->getDataService(name);
        if (raptorlet!=0) {
            RaptorDataSourceGroup  * datagroup = NULL;
                datagroup = raptorlet->data();
/*
                if (datagroup != 0) {
                                if (datagroup->count() >= 0 ) {
                                QStringList list = datagroup->getDataList();
                                
                                    for (int i = 0 ; i < list.count() ; i++) {
                                            RaptorClaw * it = new RaptorClaw(d->e);
                                            it->setText(datagroup->getItem(list.at(i))->name() );
                                            it->setIcon(KIcon ("").pixmap(64,64));
                                  //          d->view->addItem(it);
                                            kDebug()<<datagroup->getItem(list.at(i))->name()<<endl;
                                }

                }
        }
*/
    }
    }
    d->view->view(0);
}
void RaptorMenuWidget::ajustView ( const QRectF & rect ) 
{
    Q_UNUSED(rect);
}

void RaptorMenuWidget::updateSceneRect ( const QRectF & rect )
{
    Q_UNUSED(rect);
}
void RaptorMenuWidget::animate(int step )
{
    QPointF center = d->e->boundingRect().center();
    resetMatrix();
    QTransform mat = this->transform();
    mat.translate(center.x(),center.y());
    mat.scale(0.8,0.8);
    mat.rotate(step,Qt::YAxis);
    d->scene->update(QRect(0,0,1000,1000));
    mat.translate(-center.x(),-center.y());
    d->e->setTransform(mat);


}
void RaptorMenuWidget::finishAnimation()
{
    QPointF center = d->e->boundingRect().center();
    d->e->resetTransform();
    d->e->resetMatrix();
}

void RaptorMenuWidget::reloadPlugins()
{

}

#include "raptormenuwidget.moc"
