 #include <QtUiTools>
 #include <QtGui>
 #include <QFile>

 #include "tomeditorform.h"

 TomEditorForm::TomEditorForm(QWidget *parent)
     : QWidget(parent)
 {
        
     QWidget *formWidget = loadUiFile();

     ui_mainTextEdit = qFindChild<QTextEdit*>(this, "textEdit");
     ui_tree = qFindChild<QTreeWidget*>(this, "tomtree");
     ui_label = qFindChild<QLabel*>(this, "label");
     ui_labelSelected = qFindChild<QLabel*>(this, "lblSelected");

     fldName = qFindChild<QLineEdit*>(this, "fieldName");
     fldProg = qFindChild<QLineEdit*>(this, "fieldProg");
     fldDesc = qFindChild<QLineEdit*>(this, "fieldDesc");
     fldDeskF = qFindChild<QLineEdit*>(this, "fieldDeskF");

     ui_openButton = qFindChild<QPushButton*>(this, "btnOpen");
     ui_saveButton = qFindChild<QPushButton*>(this, "btnSave");
     ui_applyButton = qFindChild<QPushButton*>(this, "btnApply");
     ui_resetButton = qFindChild<QPushButton*>(this, "btnReset");

     ui_delButton = qFindChild<QPushButton*>(this, "btnDel");
     ui_editButton = qFindChild<QPushButton*>(this, "btnEdit");

     ui_addTGButton = qFindChild<QPushButton*>(this, "btnAddTG");
     ui_addTAButton = qFindChild<QPushButton*>(this, "btnAddTA");
     ui_addTIButton = qFindChild<QPushButton*>(this, "btnAddTI");

     connect( ui_tree, SIGNAL( itemSelectionChanged() ), this, SLOT( treeSelectionChanged() ) );
     //connect( ui_tree, SIGNAL( focusOutEvent() ), this, SLOT( listFocusOut() ) );
     connect( this, SIGNAL( itemListChanged() ), this, SLOT( treeSelectionChanged() ) );
     connect( ui_openButton, SIGNAL( clicked() ), this, SLOT( openFile() ) );
     connect( ui_saveButton, SIGNAL( clicked() ), this, SLOT( saveFile() ) );
     connect( ui_delButton, SIGNAL( clicked() ), this, SLOT( removeItem() ) );
     connect( ui_resetButton, SIGNAL( clicked() ), this, SLOT( resetView() ) );
     connect( ui_applyButton, SIGNAL( clicked() ), this, SLOT( btnApplyClicked() ) );
     connect( ui_addTIButton, SIGNAL( clicked() ), this, SLOT( insertItem() ) );
     connect( ui_addTAButton, SIGNAL( clicked() ), this, SLOT( insertItem() ) );
     connect( ui_addTGButton, SIGNAL( clicked() ), this, SLOT( insertGroup() ) );
     connect( ui_editButton, SIGNAL( clicked() ), this, SLOT( changeItem() ) );

     QMetaObject::connectSlotsByName(this);

     QVBoxLayout *layout = new QVBoxLayout;
     layout->addWidget(formWidget);
     setLayout(layout);

     setWindowTitle(tr("TOM Editor"));
     setFixedSize ( 770, 660 );

     ui_label->setText("Hello2");
     loadFile("/usr/share/tom/tom.xml");
 }
 
 QWidget* TomEditorForm::loadUiFile()
 {
     QUiLoader loader;

     QFile file(":forms/tom-edit.ui");
     file.open(QFile::ReadOnly);

     QWidget *formWidget = loader.load(&file, this);
     file.close();

     return formWidget;
 }
 
void TomEditorForm::about() 
{
    QMessageBox::about(this,"About myQtApp",
                "This app was coded for educational purposes.\n"
                "Number 1 is: \n\n"
                "Bye.\n");
}

void TomEditorForm::openFile()
{
     QString fileName;

     fileName = QFileDialog::getOpenFileName(this,
             tr("Open File"), "", "XML Files (*.xml)");

     if (!fileName.isEmpty()) {
         QFile file(fileName);
         if (file.open(QFile::ReadOnly | QFile::Text)){
             //ui_mainTextEdit->setPlainText(file.readAll());
             fullFilePath = fileName;
             //qDebug()<<fullFilePath;
             loadFile(fileName);
         }
     }
}

void TomEditorForm::saveFile()
{
     ManXml manXml;
     QDomDocument doc=manXml.updateXml(tomGroup);
     if(manXml.saveXml(doc,fullFilePath))
     qDebug() << "Saved!";
}

void TomEditorForm::updateTextView()
{
     ManXml manXml;
     QDomDocument doc=manXml.updateXml(tomGroup);
     ui_mainTextEdit->setPlainText(doc.toString());
}

void TomEditorForm::resetView()
{
     loadFile(fullFilePath);
}

void TomEditorForm::treeSelectionChanged()
{
  ManXml manXml;
  QList<Tom>* ptr_tomGroup = &tomGroup;

  currenti = ui_tree->currentItem();
  ui_labelSelected->setText(currenti->text(1));
  fldName->setText(currenti->text(0));

  long typeId = (currenti->text(3)).toLong();

  switch(typeId){

      case 0: //TOM Selected
          fldDeskF->setText("......");
          break;

      case 1: //TaskGrp Selected
          fldDeskF->setText("......");
          //Enable Buttons
          ui_addTAButton->setEnabled(true);
          break;

      case 2: //TaskAct Selected
          fldDeskF->setText("......");
          //Enable Buttons
          ui_addTIButton->setEnabled(true);
          //Disable Buttons
          ui_addTAButton->setEnabled(false);
          break;

      case 3: //TaskItm Selected
          //Disable Buttons
          ui_addTIButton->setEnabled(false);
          ui_addTAButton->setEnabled(false);
          int found = manXml.findTomitemChild( ptr_tomGroup, (currenti->text(2)).toLong(), FIND );
          if(found){
              TomItem tItem = manXml.getPvtItem();
              fldDeskF->setText(tItem.getDeskFilePath());
          }
          break;
  }
  //Enable Button.
  ui_delButton->setEnabled(true);
  ui_editButton->setEnabled(true);
  ui_addTGButton->setEnabled(true);
}

void TomEditorForm::loadFile( QString fileName )
{
//   if(fileName==""){
//     fileName = ":xml/tom.xml";
//     fullFilePath = fileName;
//   }

  fullFilePath = fileName;
  QDomDocument doc("tom_doc");
  QFile file(fileName);

  if(!file.exists()){
    qDebug() << "File Dsn't Exist.!.";
    return;
    //file(":xml/tom.xml");
  }

  if ( (file.open(QIODevice::ReadOnly)) && !(doc.setContent(&file)) ) {
    file.close();
  }
  else
    file.close();

  ManXml manXml;

  tomGroup = manXml.parseXMLwithDOM(doc);

  updateTextView();

  generateItemTree();
}

void TomEditorForm::generateItemTree()
{

  ui_tree->clear();
  ui_tree->hideColumn(1);
  ui_tree->hideColumn(2);
  ui_tree->hideColumn(3);

  Tom tom;
  tom = tomGroup.first();

  //qDebug("%d",tom.ID);
  tomhash.insert(tom.ID, &tom);

  QList<TomTaskGroup> tglist = tom.tomTaskGroups;
   for (int a=0; a<tglist.size(); ++a) {

      QTreeWidgetItem *tomtg_listItem = new QTreeWidgetItem(ui_tree); // Create the first node.
      TomTaskGroup tomTaskGroup = tglist.at(a);
      tomtg_listItem->setText(0, tomTaskGroup.getName());
      tomtg_listItem->setText(1, "Tom Task Group");
      tomtg_listItem->setText(2, QString("%1").arg(tomTaskGroup.ID));
      tomtg_listItem->setText(3, QString("%1").arg(1));

      taskghash.insert(tomTaskGroup.ID, &tomTaskGroup);

      QList<TomTaskAction> talist = tomTaskGroup.tomTaskActions;
      for (int b= 0; b<talist.size(); ++b) {

          QTreeWidgetItem *tomta_listItem = new QTreeWidgetItem(tomtg_listItem); // Create the first node.
          TomTaskAction tomTaskAction = talist.at(b);
          tomta_listItem->setText(0, tomTaskAction.getName());
          tomta_listItem->setText(1, "Tom Task Action");
          tomta_listItem->setText(2, QString("%1").arg(tomTaskAction.ID));
          tomta_listItem->setText(3, QString("%1").arg(2));

          taskahash.insert(tomTaskAction.ID, &tomTaskAction);

          QList<TomItem> itmlist = tomTaskAction.tomItems;
          for (int c= 0; c<itmlist.size(); ++c) {

              QTreeWidgetItem *tomitem_listItem = new QTreeWidgetItem(tomta_listItem); // Create the first node.
              TomItem tomItem = itmlist.at(c);
              tomitem_listItem->setText(0, tomItem.getDeskFilePath());
              tomitem_listItem->setText(1, "Tom Item");
              tomitem_listItem->setText(2, QString("%1").arg(tomItem.ID));
              tomitem_listItem->setText(3, QString("%1").arg(3));

              tomitemhash.insert(tomItem.ID, &tomItem);
              //qDebug() << tomItem.getName();
              //qDebug() << tomItem.getProgram();
              //qDebug() << tomItem.getDesc();
              //qDebug() << tomItem.getDeskFilePath();
          }
      }
  }
  ui_tree->expandAll();
}

void TomEditorForm::btnApplyClicked()
{
    //DCOPRef m_kickerkbfxDevelnterface ( "kicker","KBFXInterface" );
    //m_kickerkbfxDevelnterface.call ( "notifyConfigChange()" );
    //emit xmlFile_Changed();
}

void TomEditorForm::removeItem()
{
    ManXml manXml;
    QList<Tom>* ptr_tomGroup = &tomGroup;

    long typeId = (currenti->text(3)).toLong();
    int found=0;

    switch(typeId){

        case 0: //TOM Selected
            //found = manXml.findTomitemChild( ptr_tomGroup, (currenti->text(2)).toLong(), DEL );
            break;

        case 1: //TaskGrp Selected
            found = manXml.findTomtaskGrpChild( ptr_tomGroup, (currenti->text(2)).toLong(), DEL );
            break;

        case 2: //TaskAct Selected
            found = manXml.findTomtaskActionChild( ptr_tomGroup, (currenti->text(2)).toLong(), DEL );
            break;

        case 3: //TaskItm Selected
            found = manXml.findTomitemChild( ptr_tomGroup, (currenti->text(2)).toLong(), DEL );
            break;
    }

    if(found){
        currenti->setHidden(true);
        currenti->setSelected(false);
        currenti=currenti->parent();
        ui_tree->setCurrentItem(currenti);
        currenti->setSelected(true);
        updateTextView();
    }
}

void TomEditorForm::insertItem() {

  ManXml manXml;
  QList<Tom>* ptr_tomGroup = &tomGroup;
  long typeId = (currenti->text(3)).toLong();
  int found;

  switch(typeId){

      case 0: //TOM Selected
          //fldDeskF->setText("......");
          break;

      case 1: //TaskGrp Selected
          found = manXml.findTomtaskGrpChild( ptr_tomGroup, (currenti->text(2)).toLong(), FIND );
          if(found){
              TomTaskAction tAction;
              bool ok;
              QString text = QInputDialog::getText(this, tr("Add New Task Action"),
                                                    tr("New Task Action Name:"), QLineEdit::Normal,
                                                    QDir::home().dirName(), &ok);
              if (ok && !text.isEmpty()) {
                  tAction.setName(text);
                  tAction.ID=manXml.getUid();
                  manXml.incrementUid();
                  manXml.addNewAction(tAction);
              }
          }
          break;

      case 2: //TaskAct Selected
          found = manXml.findTomtaskActionChild( ptr_tomGroup, (currenti->text(2)).toLong(), FIND );
          if(found){
              TomItem tItem;
              QString fileName;
              fileName = QFileDialog::getOpenFileName(this,
                      tr("Open File"), "/usr/share/applications/", "Desktop Files (*.desktop)");
              if (!fileName.isEmpty()) {
                tItem.setDeskFilePath(fileName);
                manXml.addNewItem(tItem);
              }
          }
  }

updateTextView();
generateItemTree();
 }

void TomEditorForm::insertGroup() {

    TomTaskGroup tGroup;
    bool ok;
    ManXml manXml;
    QList<Tom>* ptr_tomGroup = &tomGroup;
    QString text = QInputDialog::getText(this, tr("Add New Task Group"),
                                          tr("New Task Group Name:"), QLineEdit::Normal,
                                          QDir::home().dirName(), &ok);
    if (ok && !text.isEmpty()) {
        tGroup.setName(text);
        tGroup.ID=manXml.getUid();
        manXml.incrementUid();
        manXml.addNewGroup(ptr_tomGroup, tGroup);
    }

updateTextView();
generateItemTree();
}

void TomEditorForm::changeItem() {

  ManXml manXml;
  QList<Tom>* ptr_tomGroup = &tomGroup;
  long typeId = (currenti->text(3)).toLong();
  int found = 0;
  switch(typeId){

      case 0: //TOM Selected
          //fldDeskF->setText("......");
          break;

      case 1: //TaskGrp Selected
          found = manXml.findTomtaskGrpChild( ptr_tomGroup, (currenti->text(2)).toLong(), FIND );
          if(found){
              bool ok;
              QString text = QInputDialog::getText(this, tr("Add New Task Action"),
                                                    tr("New Task Action Name:"), QLineEdit::Normal,
                                                    currenti->text(0), &ok);
              if (ok && !text.isEmpty()) {
                  manXml.changeGroup(text);
              }
          }
          break;

      case 2: //TaskAct Selected
          found = manXml.findTomtaskActionChild( ptr_tomGroup, (currenti->text(2)).toLong(), FIND );
          if(found){
              bool ok;
              QString text = QInputDialog::getText(this, tr("Change Task Action Name"),
                                                    tr("New Task Action Name:"), QLineEdit::Normal,
                                                    currenti->text(0), &ok);
              if (ok && !text.isEmpty()) {
                  manXml.changeAction(text);
              }
          }
          break;

      case 3: //TomItem Selected
          found = manXml.findTomitemChild( ptr_tomGroup, (currenti->text(2)).toLong(), FIND );
          if(found){
              QString fileName;
              fileName = QFileDialog::getOpenFileName(this,
                      tr("Select Desktop File"), currenti->text(0), "Desktop Files (*.desktop)");
              if (!fileName.isEmpty()) {
                  manXml.changeItem(fileName);
              }
          }
  }

updateTextView();
generateItemTree();
 }

// void TomEditorForm::focusOutEvent() {
// 
//   qDebug()<<"Focus Out.";
// }
